#############################################################################################
#   
#   REPRESENTACION ESPACIAL Y MAPAS 
#    
#   por
#
#   Rodrigo Rodrigues-Silveira
#   rodrodr@gmail.com
#   
#   APENDICE A - GUIA RAPIDA DE INTRODUCCION AL R y AL R-STUDIO
#
#   Este codigo esta destinado a ser utilizado en el paquete estadistico R
#   R version 3.0.1 (Windows). Ademas, para que funcione, todos los archivos denominados
#   "material auxiliar" deben estar presentes en la carpeta:
#   C:\\CIS\\Material Auxiliar\\
#   En el caso de que el usuario desee utilizar otra carpeta, debe alterar el 
#   contenido de la variable "cdt" - carpeta de trabajo, mas abajo.
#   
#   Ultima revision: 19 de julio de 2013.
###########################################################################################




#######################################################################################################
####################################### Pasos Iniciales en el R #######################################
#######################################################################################################


# Establece la carpeta (directorio) de trabajo (set, w - working, d - directory)
rm(list=ls())
cdt <- c("C:/CIS/Material Auxiliar/Shapes/")
setwd(cdt)
par(mar=rep(4,4))

# Crea un vector (variable) de tipo texto con el contenido "El R se aprende con poco esfuerzo"
texto  <- c("El R se aprende con poco esfuerzo")

# Imprime el contenido del vector texto
texto

# Crea un vector numerico con diferentes edades
edad <- c(5, 10, 15, 16, 19, 20, 24, 30, 34, 40, 45, 55, 59, 60, 70, 90)

# Atribuye el valor del calculo 2+2 al vector x por medio del simbolo "<-"
x <- 2+2

# obtiene los resultados
x

# Atribuye una lista de nombres a la variable nombres por medio de la funcion c(.) 
# y el simbolo <-
nombres <- c("Carmen", "Antonio", "Maria", "Jos?")

# Reproduce la creacion del vector edad
edad <- c(5, 10, 15, 16, 19, 20, 24, 30, 34, 40, 45, 55, 59, 60, 70, 90)

# Emplea los operadores logicos >= (mayor o igual), & ("y" logico), y <= (menor o igual)
# para crear un vector (adulto) que corresponde al subconjunto del vector edad, 
# conteniendo solamente las edades entre 20 y 60 a?os.
adulto <- edad[edad >= 20 & edad <= 60] 

# Exhibe el subconjunto de edades entre 20 y 60 anos.
adulto    

# Funcion mean(var) calcula la media de una determinada variable
mean(edad)   

# Funcion boxplot, genera una diagrama de cajas para las edades, en horizontal
boxplot(edad, horizontal=TRUE)   


#######################################################################################################
###################### Operaciones con bases de datos y calculo de variables ##########################
#######################################################################################################

# establece la carpeta de trabajo, que contiene todos los ejemplos del manual
setwd("C:/Rodrigo/Stats/CIS/Shapes/")

# abre el archivo "datos_ccaa.csv" en el objeto de tipo base de datos de nombre datos
# Parametros:
#              "datos_ccaa.csv" - nombre del archivo que se desea abrir
#              header=TRUE - considera la primera linea como el nombre de las variables
#              sep=";" - define el separador de variables como punto y coma ";"
datos <- read.csv("datos_ccaa.csv ", header=TRUE, sep= ";")

# Lista todas las variables pertenecientes en el objeto datos
names(datos)

# El simbolo $ se emplea para referirse a una variable - data.frame$variable:
datos$pob09

# El mismo razonamiento vale para cualquier calculo y creacion de nuevas variables en la
# base de datos, como la creacion de la variable pib_total:
datos$pib_total <- datos$pibpct09 * datos$pob09

# Una forma de facilitar el analisis es emplear la funcion attach(.) que permite que el R
# considere la base de datos referida como la principal, sin necesidad de tener que siempre
# mencinarla cuando se invoque una variable
attach(datos)

# lista el contenido de la variable ccaa_nombre (nombre de las Comunidades Autonomas) 
# contenida en la base de datos "datos"
ccaa_nombre

# Para remover la referencia automatica a una base de datos basta utilizar la funcion 
# detach(.)
detach(datos)

# Al hacerlo el R no reconocera automaticamente la variable
ccaa_nombre

# Repite el ejemplo de la funcion attach(.) para demostrar sus limitaciones.
attach(datos)

#Abajo se encuentra la forma equivocada del uso de attach(.), pues la variable pib_total 
# sera creada, pero no pertenecera a la base de datos datos
pib_total <- pibpct09 * pob09   	
pib_total			      # examina el contenido de la variable pib_total
rm(pib_total) 			# remueve la variable de la memoria

# La forma correcta para crear la variable en la base de datos seria:
datos$pib_total <- pibpct09 * pob09

# Llamando variables antiguas y nuevas a partir de attach(.)
pob09 		        # Retorna una lista con la poblacion de las 19 regiones
pib_total		      # Retorna un error indicando que este objeto no existe
datos$pib_total		# Ahora retorna la lista con el PIB total de las 19 regiones

# Ejecutando nuevamente la funcion attach(.)
attach(datos)
pib_total		# Retorna la variable, pues esta ya esta incorporada a la base de datos

# Para guardar los resultados o cambios en la base de datos, se debe emplear la funcion
# write.table(.)
# Parametros:
#             datos - base de datos que se desea guardar en un archivo
#             "datos_ccaa.csv" - nombre del archivo
#             sep=";" - separador utilizado para distinguir las variables
write.table(datos, "datos_ccaa.csv", sep=";")


#######################################################################################################
################################### Instalando y cargando paquetes ####################################
#######################################################################################################

# Para instalar un paquete estadistico en el R se debe usar la funcion install.packages() 
# con el nombre del paquete que se desea instalar.
install.packages("maptools")

# Para cargar un paquete se necesita la funcion library()
library("maptools")



#######################################################################################################
########################## Programacion en el R y funciones del usuario ###############################
#######################################################################################################


## Para mas informacion, ver: http://en.wikibooks.org/wiki/R_Programming

# A) Estructuras de Controle (http://www.statmethods.net/management/controlstructures.html)

# If (si) - trabajando con condiciones 

# Genera dos variables utilizadas para realizar un teste logico
a <- 2
b <- 0

# If (se) A es mayor que 1 y B es menor que 1, imprima el mensaje "Condicion satisfecha!", else (caso contrario) imprima el mensaje "Valor de a<2 o de b >= 1 !"
if (a >1 & b < 1) print("Condicion satisfecha!") else print("Valor de a < 2 o de b >=1 !")


# Trabajando con loops (repeticiones de procedimientos) - lapply, for y while

# Algunas vezes resulta necesario repetir procedimientos con el objetivo de completar las tareas necesarias. En estos casos, se puede utilizar algunos procedimientos para crear ciclos o repeticiones en los comandos. Existen en R tres funciones basicas:  lapply, for y while.

# Genera una variable denominada val con 5 letras
val <- c("A", "B", "C", "D", "E")

# Usa la funcion lapply() para imprimir cada valor de la variable 
# lapply(), aplica la misma funcion (print) para cada uno de los elementos de la variable val
lapply(val, function(x){print(paste("el valor de la variable es: ", x, sep=" "))})

# Usa la funcion for() para imprimir cada valor de la variable
# for() genera un contador de 1 al tamano de los elementos de val (n=5) e imprime el valor de cada uno de ellos
for(i in 1:length(val)) print(paste("el valor de la variable es: ", val[i], sep=" "))

# Utiliza la funcion while() para imprimir cada valor de la variable
# Los siguientes comandos generan un contador regresivo que imprime los valores de val en orden inversa
i <- length(val)
while(i > 0){
  print(paste("el valor de la variable es: ", val[i], sep=" "))
  i <- i-1
}



# B) Funciones del usuario (http://www.statmethods.net/management/userfunctions.html)


# Como las funciones pre-programadas, los usuarios de R tambien pueden programar funciones propias, establecendo comandos y parametros que crean necesarios para realizar un conjunto de operaciones que se repitan o que sean de uso corriente, como por ejemplo el calculo del numero efectivo de partidos o la creacion de un mapa de coropletas. 

# Parameters: specific instructions necessary to the execution of a set of actions.


# Funcion enep() - calcula el numero ecfetivo de partidos electorales. Utiliza un solo parametro: umn variable con los votos de cada partido en una unidad territorial.
enep <- function(voto){
  
  # Verifica si los valores de la variable se encuetran en proprociones o porcentajes
  ln <- which(voto>1)
  
  # Si algun caso es mayor que 1 (el equivalente a 100% en el formato de proporcion), divide el contenido de la variable por 100
  if (length(ln)>0) voto <- voto/100
  
  # Calcula el indicador y retorna el valor final
  return(1/sum(voto^2))
  
}

# Para testar la funcion se genera una variable con la votacion hipotetica de cuatro partidos (en porcentaje).
voto_partidos <- c(5, 10, 30, 55)

# Ejecuta la funcion enep(.) que calcula el numero efectivo de partidos electorales
enep(voto_partidos)



#######################################################################################################
######################################## Parametros graficos ##########################################
#######################################################################################################

### A) La funcion par(.)

# Exibe una lista de todos los parametros graficos y sus valores predefinidos
par()

# Accede a la ayuda de la funcion par() que contiene la definicion de la mayor parte de los parametros
help(par)


# Ejemplo: alterando los margenes de un grafico
plot(1,1)

# Altera los margenes para dos lineas en cada lado
par(mar=c(2,2,2,2))
plot(1,1)

# Altera el color de los ejes para rojo
par(col.axis="red")
plot(1,1)

# Altera el formato del punto para un triangulo
plot(1,1, pch=17, cex=3, col="blue")

# Restaura los valores antiguos
par(mar=c(4,4,4,4))
par(col.axis="black")
plot(1,1)



# Genera un plano cartesiano en el R
plot(seq(-5,5),seq(-5,5), col=c(rep("red",5),"darkgreen",rep("blue",5)), pch=19)

# La funcion abline dibuja dos rectas una horizontal y otra vertical,que se cruzan en el valor cero.
abline(h=0,v=0)


# Ejemplor de grafico utilizando plot para determinar coordinadas
edad <- c(10,15,20,40,60)
peso <- c(40,60,75,80,85)
plot(x=edad,y=peso, pch=19)


# main= define el titulo del grafico
plot(1,1, main="Titulo del grafico")

# xlab= e ylab= definen los nombres de los ejes x e y
plot(1,1, main="Titulo del grafico", xlab="variable X", ylab="variable Y")

# pch= define el formato del punto que se esta utilizando, en este caso utilizaremos el circulo pch=19
plot(1,1, main="Titulo del grafico", xlab="variable X", ylab="variable Y", pch=19)

# col= define el color del punto, linea, curva, o cualquier outro objeto dibujado
plot(1,1, main="Titulo del grafico", xlab="variable X", ylab="variable Y", pch=19, col="red")

# cex= define el factor de expansion del tamano de los caracteres. Cuanto mayor sea el cex, mayores seran los objetos (en este caso el punto)
plot(1,1, main="Titulo del grafico", xlab="variable X", ylab="variable Y", pch=19, col="red", cex=2)

# cex.main= define el tamano del titulo
plot(1,1, main="Titulo del grafico", xlab="variable X", ylab="variable Y", pch=19, col="red", cex=2, cex.main=2)



#######################################################################################################
########################################## Obteniendo ayuda ###########################################
#######################################################################################################


# busca la palabra mean en todos los paquetes instalados en el R
??mean

# Abre la ayuda referente a la funcion mean 
help(mean)

# Abre la descripcion del paquete maptools
help(package="maptools")

# Busca la palabra "maps" en toda la documentacion del R
help.search("maps")

# Abre un manual mas descriptivo de un determinado paquete (cuando existente) 
vignette(package="sp", all=T)





#####################################################################################################
################################## FIN DEL ARCHIVO DE LOS APENDICES #################################
#####################################################################################################
