#############################################################################################
#   
#   REPRESENTACION ESPACIAL Y MAPAS 
#    
#   por
#
#   Rodrigo Rodrigues-Silveira
#   rodrodr@gmail.com
#   
#   PRIMER CAPITULO - EJEMPLOS
#
#   Este codigo esta destinado a ser utilizado en el paquete estadistico R
#   R version 3.0.1 (Windows). Ademas, para que funcione, todos los archivos denominados
#   "material auxiliar" deben estar presentes en la carpeta:
#   C:\\CIS\\Material Auxiliar\\
#   En el caso de que el usuario desee utilizar otra carpeta, debe alterar el 
#   contenido de la variable "cdt" - carpeta de trabajo, mas abajo.
#   
#   Ultima revision: 19 de julio de 2013.
###########################################################################################

## INSTRUCCIONES GENERALES

# A - Instalacion del R

# El paquete estadistico R puede ser obtenido de forma gratuita en las siguientes direcciones:
# Para WINDOWS: http://cran.r-project.org/bin/windows/base/
# Para MAC-OS X: http://cran.r-project.org/bin/macosx/
# Para LINUX: http://cran.r-project.org/bin/linux/

# En estas mismas paginas web se encuentran las instrucciones para realizar
# la instalacion del R en el ordenador deseado.


# B - Instalacion de los PAQUETES R que seran empleados

# El R trabaja con paquetes desarrollados por investigadores independientes
# que posibilitan una gran oferta de tecnicas y estadisticas al mismo tiempo
# que hacen del programa mas flexible y adaptable a cambios.
# La necesidad de instalar y cargar los paquetes estadisticos evita que el 
# programa consuma demasiada memoria y espacio fisico con instrumentos
# que muchas veces no son necesarios para el investigador. El resultado
# son procesos mas rapidos.

# Para instalar un paquete en R el usuario debe:
# (a) iniciar el programa
# (b) ir al menu "Paquetes"
# (c) elegir la opcion "Instalar Paquetes"
# Aparecera un menu con un listado de servidores
# (d) elegir el servidor que esta mas cerca (en el caso Spain - Madrid)
# Aparecera un menu con un listado de todos los paquetes disponibles on-line
# (e) seleccionar el(o los) paquete(s) deseado(s) y clicar en OK
# El R instalara los paquetes deseados.

# Otra opcion es utilizar la funcion install.packages("nombre del paquete")



# C - Instalacion del R-Studio

# Infelizmente, el editor de codigo fuente del R es muy limitado visualmente.

# Para subsanar esa limitacion, se recomienda la utilizacion del aplicativo
# R-STUDIO, disponible de forma gratuita en: http://www.rstudio.org/
# La ventaja del R-Studio es que permite manejar el R de modo mucho mas "amigable"
# En el es relativamente facil editar el codigo-fuente, instalar paquetes y visualizar graficos y datos.


# D - Instrucciones sobre el CoDIGO FUENTE

# Este codigo-fuente, que consiste en el conjunto de comandos utilizados 
# por el R para llevar a cabo cualquier tarea especifica, posee dos objetivos
# principales:
# (1) facilitar el aprendizaje de los usuarios poco familizarizados con
#     el R y permitir que tengan la oportunidad de utilizar los ejemplos descriptos en el 
#     manual como forma de acercarse al programa y familiarse con su utilizacion.
# (2) permitir a todos los lectores llevar a cabo la replicacion de los 
#     ejemplos mencionados en los capitulos para saber exactamente
#     "como se hizo". 

# Para que el proceso de comprension y acercamiento al codigo-fuente y a los 
# procesos de construccion de los ejemplos, la decision ha sido insertar un
# comentario explicativo todas las veces que aparece un nuevo comando. Asi, 
# el lector estara familiarizado con los principales comandos utilizados en
# R para el analisis espacial y los parametros empleados para obtener los 
# resultados esperados. Muchas veces, los comentarios del codigo-fuente seran
# mas extensos y completos que aquellos contenidos en los capitulos del manual,
# por lo que se sugiere el empleo de ambos simultaneamente.

# En el codigo-fuente, el simbolo "#" corresponde a una marca de comentario. Siempre que una 
# linea sea iniciada por esta marca, esto significa que se comenta o da nombre al comando que 
# viene a continuacion.

# E - COPIA del MATERIAL AUXILIAR

# Se recomienda la copia de los archivos que acompanan este volumen en la
# carpeta de trabajo C:\\CIS\\Material Auxiliar\\
# El usuario puede elegir otra segun su conveniencia o gusto. En este caso,
# debe cambiar los valores de la variable "cdt" en todo el codigo-fuente.
# En caso contrario, ninguno de los ejemplos aqui descritos funcionara.


# F - AYUDA y DOCUMENTACIoN sobre el R

# REGLAS BaSICAS: una guia basica de como utilizar el R se puede encontrar
#                 en los apendices de este volumen. No se trata de algo
#                 exhaustivo, pero sirve para familiarizar el lector con
#                 el lenguaje basico del programa.

# DOCUMENTACIoN:  El R posee un volumen bastante grande de informacion disponible 
#                 en Internet. La pagina web del Proyecto CRAN - 
#                 http://cran.r-project.org/ posee varios manuales y 
#                 documentacion adicional sobre el R en general y cada uno de los
#                 paquetes en particular.

# AYUDA:  para obtener ayuda sobre cualquier comando del R basta digitar 
#         help(comando), el programa automaticamente abrira el navegador 
#         de Internet y le redireccionara a una pagina con la descripcion del 
#         comando deseado, los parametros utilizados y alguns ejemplos de su utilizacion.

# OBS.: La mayor parte de la documentacion del R se encuentra en el idioma
#       ingles. Por esa razon, es fundamental que el lector posea una buena
#       comprension de esta lengua para hacer uso pleno de las potencialidades
#       de la herramienta.


##############################################################################################
################################ EJEMPLOS - CAPITULO 1 ####################################### 
##############################################################################################


## "Conceptos generales e instrumentos para la representacion visual de fenomenos sociales en el espacio


# Establece la carpeta de trabajo (espacio donde estan ubicados los archivos auxiliares)
# La funcion setwd(.) es la responsable por establecer el directorio (o carpeta) de trabajo
# El unico parametro (informacion) necesario es la indicacion de que carpeta se utilizara
# en este caso la que contiene la variable cdt, es decir, "C:\\CIS\\Material Auxiliar\\Shapes\\"
cdt <- c("C:/CIS/Material Auxiliar/Shapes/")
setwd(cdt)

# Como el manual emplea un volumen extensivo de paquetes, se necesitan instalarse por medio de los siguientes comandos:

# A - se crea una lista de paquetes a ser instalados
lista <- c("RColorBrewer","maptools", "maps","mapproj","rgdal", "PBSmapping","GeoXp", "spatstat","sp", "VIM", "classInt", "diagram","RODBC","foreign","Hmisc","fields", "spdep", "RANN","DCluster","raster", "gdistance","XML", "RgoogleMaps", "tripack","deldir","alphahull", "rgeos", "plotKML","OpenStreetMap","qualityTools")

# B - instala los paquetes contenidos en la lista definida arriba
install.packages(lista, dependencies=TRUE)

# Carga a la memoria los paquetes de R necesarios para la representacion espacial de datos
# El comando library(nombre del paquete) - realiza la carga del paquete deseado.
# Obs.: Si estos paquetes no se encuentran instalados, el usuario debe instalarlos antes
# de utilizar los comandos abajo descritos. Ver Item B en las Instrucciones Generales
# mas arriba
library(maptools)
library(maps)


# SECCION 1.2. - PROYECCIONES GEOGRAFICAS


# Mapa de las comunidades espanolas segun tres proyecciones distintas (figura 1.1)

# redefine los parametros para los graficos para que los margenes sean menores
par(mar=c(0,0,0,0.1))

# Abre el archivo Shape con las comunidades autonomas espanolas
# La funcion readShapeSpatial(.) permite leer archivos de tipo Shape (con las formas de los mapas)
# en el interior del R a un objeto de tipo SpatialShape. Los parametros son los siguientes:
# nombre - nombre del archivo shape a ser abierto (por ejemplo "ccaa.shp")
ccaa <- readShapeSpatial("ccaa.shp")

# Excluimos Canarias del mapa, para facilitar la comparacion entre proyecciones
ccaasin <- ccaa[ccaa$NOMBRE_CCA != "Canarias",]

# Divide la pantalla en tres partes para situar los tres mapas lado a lado
par(mfrow=c(1,3))

# Mapea las comunidades autonomas (sin Canarias) sin la atribucion de una proyeccion geografica
plot(ccaasin)

# Atribuye una proyeccion por primera vez - longitud/latitud y WGS84 para la elipsoide (la misma de Google Earth y Google Maps) 
proj4string(ccaasin) <- CRS("+proj=longlat +ellps=WGS84")

# Mapea las comunidades autonomas con la proyeccion Lat/Long
plot(ccaasin) 

# Carga la biblioteca rgdal que permite el cambio de proyeccion
library(rgdal)

# Proyeccion UTM comunmente utilizada en Espana
ccaasin <- spTransform(ccaasin,CRS("+proj=utm +zone=30 +ellps=GRS80 +units=km +no_defs"))

# Mapea las comunidades autonomas con la proyeccion UTM Zona 30 Norte
plot(ccaasin)

# restaura la configuracion inicial de la ventana de graficos
par(mfrow=c(1,1))




# Preparacion para la FIGURA 1.2

# Genera una variable con el tipo de proyeccion que sera utilizada en todo el capitulo
#(Proyeccion: Latitud y Longitud; Datum: WGS84)
# En general las informaciones sobre la proyeccion de un mapa son suministradas
# por quienes lo construyeron. La mayor parte de los sitios web que disponibilizan
# los mapas tambien suministran la proyeccion. En caso negativo, los archivos con
# extension .prj contienen las proyecciones geogr?ficas y muchas veces acompanan
# los archivos de tipo .shp . Los archivos de tipo .prj pueden abrirse utilizando el 
# bloc de notas o cualquier editor de texto.
proyeccion <- "+init=epsg:3035"


# Abrir los archivos de mapas en el R (funcion: readShapeSpatial(.))
# La funcion readShapeSpatial(.) permite leer archivos de tipo Shape (con las formas de los mapas)
# en el interior del R a un objeto de tipo SpatialShape. Para abrir un archivo shape se necesita solamente
# el nombre del archivo, pero tambien se puede abrirlo definiendo de antemano la proyeccion y otros parametros
# segun las necesidades del investigador. Los principales parametros son los siguientes:
# nombre - nombre del archivo shape a ser abierto (por ejemplo "ccaa.shp")
# proyeccion - tipo de proyeccion geografica, en el caso la contenida en la variable proyeccion
# IDvar - variable en el archivo dbf asociado al shp (con el mismo nombre) que identifica
#         cada unidad espacial de forma unica.
# repair = TRUE - valido para el caso de shapefiles generados por algunos programas que no
#                 son totalmente compatibles con el formato ESRI (estandar)
# force_ring = TRUE - el mismo que la opcion anterior, resuelve problemas de creacion en el
#                     archivo shape, en particular cierra poligonos que no han sido cerrados
#                     completamente.

# Abre el archivo de mapas de las provincias espanolas en el objeto "provs"
library(maptools)

# A) modo sencillo
provs <- readShapeSpatial("provincias.shp")

# B) modo mas completo, definiendo la proyeccion, la variable que identifica los casos y otros parametros de abertura
provs <- readShapeSpatial("provincias.shp", proj4string = CRS(proyeccion), IDvar = "ID_2", repair=TRUE, force_ring=TRUE)


# Visualiza el mapa de las provincias espa?olas (figura 1.3 - area)

# La funcion plot(.) genera el mapa
# plot(objeto a ser representado graficamente)
plot(provs)

# genera el mapa de las provincias de Espana continental (sin las Islas Canarias)
provsin <- provs[provs$NAME_1 != "Islas Canarias",]
plot(provsin)


# Abre el archivo de mapas de los ferrocarriles espanoles en el objeto "ffcc" (lo mismo que el ejemplo anterior)
ffcc <- readShapeSpatial("ESP_rails.shp")
ffcc <- readShapeSpatial("ESP_rails.shp", proj4string = CRS(proyeccion),  IDvar = "FID_rail_d",repair=TRUE)


# Dibuja el mapa de los Ferrocarriles espanoles en AZUL (figura 1.3 - Linea)

# el parametro "col =" permite la definicion del color del grafico o mapa
# la funcion "color()" suministra un listado completo de los colores de R
# Una lista detallada y muy util de los colores y de los nombres puede ser  
# encontrada en el sitio web: 
# http://research.stowers-institute.org/efg/R/Color/Chart/ColorChart.pdf
plot(ffcc, col = "blue")
                   

# Abre el archivo de mapas de las capitales de las provincias
caps <- readShapeSpatial("capitals.shp")
caps <- readShapeSpatial("capitals.shp", proj4string = CRS(proyeccion), IDvar = "OBJECTID",repair=TRUE)

# Dibuja las capitales de las provincias espanolas (figura 3 - Puntos)
# en ROJO (col="red"), como circulos (pch=19) y
# con un tamano de los puntos relativamente grandes para visualizacion (cex=2) 
plot(caps, col = "red", pch=19, cex=2)


# SUPERPOSICIoN DE CAPAS - Las VISTAS

# Para obtener una vista es necesario que se superpongan cada uno de las capas
# que han sido mapeados o dibujados de forma aislada en el procedimiento anterior.

# El argumento "add=TRUE" de la funcion plot(.) permite que el mapa anterior 
# no se pierda cuando uno nuevo es dibujado y, por tanto, haya superposicion.

# VISTA DE LAS PROVINCIAS, FFCC Y CAPITALES DE ESPANA (figura 1.4) 

mat <- matrix(c(1,4,4,2,4,4,3,4,4),nrow=3,ncol=3,byrow=T)
layout.show(layout(mat))
par(mar=rep(0,4))

plot(provsin)
box()

plot(ffcc)
box()

plot(caps)
box()

par(mar=rep(0.1,4))

# Dibuja las fronteras de las provincias (mapa base del cartograma)
plot(provsin, lwd=2)
# Anade en AZUL los FFCC espanoles al mapa de las provincias (add=TRUE)
plot(ffcc, col = "blue", add = TRUE)

# Anade a las dos capas anteriores las capitales de las provincias (add=TRUE)en ROJO
plot(caps, col = "red", pch=19, cex=1, add = TRUE)


# SUPERPOSICIoN DE CAPAS - EJEMPLO con cines, densidad demografica y alfabetizacion 
# en Sao Paulo (Brasil)

# Reestablece los parametros graficos
par(mfrow=c(1,1))

# Abre el archivo de mapas (area) de los barrios de la ciudad de Sao Paulo en Brasil
sp <- readShapeSpatial("distritos_SP.shp")

# Abre el archivo de mapas (puntos) de los cines
cines <- readShapeSpatial("SP_Cinemas.shp")

# Calcula la tasa de alfabetizacion del municipio de Sao Paulo
# sp es el nombre del objeto de mapa
# el simbolo ($) se emplea para referirse a una variable o columna en la
# base de datos adjunta al mapa. En este caso se utilizan dos variables:
# sp$ALFABETIZA (alfabetizados) y sp$POPULACAO (poblacion)
# para obtener un listado del nombre de todas las variables del objeto sp se debe
# utilizar la funcion "names(sp)".
sp$ts_alfa <- sp$ALFABETIZA / sp$POPULACAO * 100


# CREACION DE FRANJAS que dividen los casos de las variables de interes

# Abajo, el comando round(var, digits=N) se utiliza para redondear el numero de 
# decimales en este caso dos digitos - round(var, digits=2) o round(var, 2)

# El comando quantile(var, puntos de corte, na.rm) se utiliza para dividir los
# casos de la variable de interes en porcentajes. En el caso de ese ejemplo
# se divide la distribucion de las variables densidad demografica (DENSMO_KM2)
# y tasa de alfabetizacion (ts_alfa) en cuatro grupos con 25% de los casos:
# El parametro probs es que define los intervalos - por el comando seq(0,1,0.25),
# que representa un inicio en 0, final en 1 (100%), con progresion en la escala
# de valores de 0.25 (25%). Si uno quisiera dividir en tres bastaria definir
# la escala de progresion como 0.33, por ejemplo.
# La opcion na.rm=T (TRUE) significa que, para el calculo de los puntos de corte
# en la distribucion de la variable, los casos perdidos (o missing) son excluidos.
# Eso permite que tengamos franjas representativas de la variacion real en el 
# valor de los casos.
franjas <- round(quantile(sp$DENSMO_KM2, probs=seq(0,1,0.25),na.rm=T), digits=2)
fran_alf <- round(quantile(sp$ts_alfa, probs=seq(0,1,0.25),na.rm=T), digits=2)

## Obs.: En el capitulo 3 se discutira con mas detalle los diversos modos que  
##       pueden ser utilizados para crear franjas y representar intervalos de datos.


# Establece la escala de colores que sera utilizada para estas franjas
# La funcion "c(.)", a su vez, es una funcion generica con el objetivo de 
# combinar argumentos o parametros. Resulta especialmente util cuando se 
# necesita pasar un numero grande de argumentos para una sola funcion o parametro,
# como los colores que serviran de simbolo para cada una de las franjas
colores <- c("white", "darkslategray1", "dodgerblue", "blue4")

# INTERESANTE:
# El paquete RColorBrewer posee un conjunto de escalas de colores pre-definidas
# que pueden ser muy utiles a la hora de visualizar los datos.
# Para mayor informacion sobre el empleo del color en los mapas y graficos 
# ver la seccion 3.2b en el tercer capitulo

# Mapa de la DENSIDAD DEMOGRAFRICA (figura 1.5) 

# nuevos parametros - funcion "findInterval(.) - relaciona los intervalos de color de
# la variable DENSMO_KM2, creados a partir de la variable franjas (ver mas arriba)
# para todos los valores.
# El parametro "border=" define el color de la borda del mapa. En este caso,
# se ha elegido el color negro (Black). En el caso de que no se desee visualizar
# la borda, se debe utilizar el color "transparent".
plot(sp, col=colores[findInterval(sp$DENSMO_KM2, franjas, all.inside=TRUE)], border = "black")

# Anade en Rojo los cines (add=TRUE)
plot(cines, col = "red", pch=19, cex=1, pt.cex = cex, add = TRUE)


# CREACION de la LEYENDA del mapa
# La funcion as.character(var) convierte a caracteres alfanumericos las 
# franjas de valor para la variable de interes.
# La secuencia de valor 2:5 define que no se desea conocer el 
# valor minimo de la distribucion (Franjas[1]).
frj_char <- as.character(franjas[2:5])

# La funcion legend() crea la leyenda del mapa
# Parametros:
# "bottomright" - es la posicion en el grafico en que estara ubicada la leyenda
#                 en este caso sera en el canto inferior derecho del mapa. Para
#                 conocer todas las opciones, ver los ejemplos de help(legend)
# legend - son los valores que daran nombre a las franjas (frj_char)
# fill - los colores utilizados en la leyenda.
# cex - el factor de expansion del caracter, define las proporciones de las
#       leyendas en relacion al mapa en genera
# pt.cex - lo mismo que cex, pero para los puntos (o cajas).
# title - titulo de la leyenda.
legend("bottomright", legend = frj_char, fill = colores, cex= 0.8, pt.cex = cex, title = "Hab./km2")


# TASA DE ANALFABETISMO (figura 1.5)
plot(sp, col=colores[findInterval(sp$ts_alfa, fran_alf, all.inside=TRUE)], border = "black")

# Anade en Rojo los cines (add=TRUE)
plot(cines, col = "red", pch=19, cex=1, pt.cex = cex, add = TRUE)
                       
# Convierte a caracteres alfanumericos las franjas de valor para la variable de interes
frj_alf <- as.character(fran_alf[2:5])
                       
# Crea la leyenda del mapa
legend("bottomright", legend = frj_alf, fill = colores, cex= 1, pt.cex = cex, title = "% alfab. (2000)")






# SECCION 1.3 - MAPAS TEMATICOS


############ Mapa Tematico - COROPLETICO (figura 1.6) ############

# Abre el archivo de mapas de las Comunidades Autonomas
ccaa <- readShapeSpatial("ccaa.shp")
ccaa <- ccaa[ccaa$NOMBRE_CCA!="Canarias",]


# genera las 4 franjas para la variable de interes: 
# (POB_PER - porcentaje de la poblacion total de Espana)
franjas <- round(quantile(ccaa$POB_PER, probs=seq(0,1,0.25),na.rm=T), digits=2)

# Convierte a caracteres alfanumericos las franjas de valor para la variable de interes
leg_fr <- as.character(franjas[2:5])

# Define la escala de colores (se empleara una escala de GRISES)
# Una guia con la escala completa de colores para el R puede encontrarse en:
# http://research.stowers-institute.org/efg/R/Color/Chart
grises <- c("white", "gray85", "gray55","gray20")

# Dibuja el mapa del porcentaje de Poblacion de las CCAA 
plot(ccaa, col=grises[findInterval(ccaa$POB_PER, franjas, all.inside=TRUE)], border = "black")

# Crea la leyenda del mapa (a la izquierda del mapa)
legend("bottomright", legend = leg_fr, fill = grises, cex= 1, pt.cex = cex, title = "% de la poblacion")


############ Mapa Tematico - SIMBOLOS PROPORCIONALES (figura 1.7) ############

# Convierte el mapa de poligonos de las comunidades autonomas en a un conjunto de puntos (centroides)
ccaapt <- SpatialPointsDataFrame(ccaa, data.frame(ccaa))

# Define el tamano maximo del tamano del punto a ser representado
nmax <- 7

# Calculo del tamano de cada punto (con base en el tamano maximo de punto previamente definido para cex)
# Dos tipos:  Matematico - cada cual es representado por su valor relativo de la variable en cuestion
#             Perceptual - Reduce el peso visual de los valores mas grandes
ccaapt$size <- (ccaapt$POBLACION/max(ccaapt$POBLACION, na.rm=T))^0.5*nmax       # Matematico
ccaapt$size <- (ccaapt$POBLACION/max(ccaapt$POBLACION, na.rm=T))^0.57*nmax      # Perceptual

# Calculo dos tamanhos, da legenda e da ordem de desenho dos elementos
legs <- sort(round(quantile(ccaapt$POBLACION,seq(0,1,1/3), na.rm=T),0),decreasing=T)
ri <- (legs/max(legs, na.rm=T))^0.5*nmax      # Matematico
ri <- (legs/max(legs, na.rm=T))^0.57*nmax # Perceptual

# Establece el orden de diseno de los mayores a los menores para evitar que la superposicion oculte puntos.
ccaapt$plotOrder<- order(sort(ccaapt$size, decreasing=T))

# Dibuja el mapa de las comunidades autonomas
plot(ccaa, col="wheat1")
plot(ccaapt, cex=ccaapt$size, col="black", bg="red", pch=21, add=T)

# Formata los valores para que sean mas legibles
legs <- format(legs, big.mark=".")
legend("bottomright", legend=legs[1:length(legs)-1], col="black", pt.bg="red",pch=21,pt.cex=ri[1:length(ri)-1], cex=2, bty="n", bg="transparent", y.intersp=1, x.intersp=1)





############ Mapa Tematico - DENSIDAD DE PUNTOS (figura 1.8) ############

# Abre el archivo de mapas de las escuelas de Sao Paulo
esc_est <- readShapeSpatial("esc_estadual.shp")

# Mapea en Azul las escuelas
plot(esc_est, col = "blue", pch=19, cex=0.5, pt.cex =1, xlim=bbox(sp)[1,], ylim=bbox(sp)[2,])

# Anade el mapa de las fronteras de los barrios de Sao Paulo
plot(sp, add=TRUE)



############ Mapa Tematico - ISOPLETICO (figura 1.9) ############


# Como el mapa de simbolos proporcionales, el mapa ISOPLETICO requiere la realizacion
# de pasos previos antes de la visualizacion final de los resultados. Se utilizaran
# aqui los mapas ya abiertos de barrios de Sao Paulo (sp), escuelas (esc_est) y
# cines (cines).
# Tambien aqui son tres los pasos:
# (1) Genera el patron de difusion de las nubes de puntos para las escuelas y cines
#     con el objetivo de identificar los puntos centrales y establecer una gradacion
#     decreciente para su intensidad o densidad de concentracion. Para eso se
#     utilizara el algoritmo de "kernel" que determina la densidad de la distribucion
#     de los puntos en un mapa.
# (2) Se dibuja la rejilla (GRID) de colores progresivos que va de colores mas calientes
#     en la zona con mas densidad de puntos hasta colores mas frios en las zonas
#     de menor densidad (es decir, con menor presencia de escuelas o cines)
# (3) Como en el ejemplo de los simbolos proporcionales, se mapean los limites de 
#     los barrios de Sao Paulo para que el lector conozca la densidad de las escuelas
#     o cines por barrio. En este caso, se hace despues para que las fronteras 
#     no sean superpuestas.


# KERNEL 

# Genera una rejilla (GRID) por medio de la cual se podra representar la funcion
# segun variaciones que no respetan los limites de los poligonos

# la funcion spastat.options(.) deshabilita la opcion de averiguar la superposicion de
# poligonos a la hora de crear la rejilla. En el momento posterior, una vez creada
# la rejilla, se reactiva la opcion. 
library(spatstat)
spatstat.options(checkpolygons=FALSE)

# La funcion as.owin.SpatialPolygons, genera la rejilla a ser utilizada en la funcion
# Kernel segun el mapa de barrios de la ciudad de Sao Paulo.
sp_owin <- as.owin.SpatialPolygons(sp)

# reactivacion de la opcion de averiguacion de superposicion de poligonos.
spatstat.options(checkpolygons=TRUE)


# Obtiene las coordenadas de las escuelas de Sao Paulo
cord_esc <- coordinates(esc_est)

# Obtiene las coordenadas de los cines de Sao Paulo
cord_cine <- coordinates(cines)

# Genera los objetos de puntos que serviran para representar los mapas ISOPLETICOS
# la funcion as.ppp(.) necesita de dos parametros - las coordinadas y la ventaja (W - window)
# que es como una forma que establece los limites para los puntos.
pt_esc <- as.ppp(cord_esc, W=sp_owin)
pt_cine <- as.ppp(cord_cine, W=sp_owin)

# Calcula la funcion de densidad de Kernel para los dos archivos de puntos
# density(.) - objeto de puntos y "n=" numero de estimaciones para la gradacion
#             en el mapa.
kr_esc <- density(pt_esc, n=1000)
kr_cine <- density(pt_cine, n=1000)


# Genera el MAPA ISOPLETICO para ESCUELAS (figura 1.9)

# Mapea la funcion de densidad de Kernel para las Escuelas
# topo.colors(12) - funcion de colores pre-programada de R que representa una
#                   escala topografica de colores. Otras funciones son:
#                   rainbow(n), heat.colors(n), terrain.colors(n), cm.colors(n)
# main= - titulo del grafico. 
plot(kr_esc, col = topo.colors(12), main="Escuelas (kernel)")

# Anade al mapa anterior las fronteras de los barrios de Sao Paulo
plot(sp, add=T)


# Genera el MAPA ISOPLETICO para CINES (figura 1.9)

# Mapea la funcion de densidad de Kernel para los CINES
plot(kr_cine, col = topo.colors(12),main="Cines (kernel)")

# Anade al mapa anterior las fronteras de los barrios de Sao Paulo
plot(sp, add=T)





############ Mapa Tematico - CARTOGRAMA - ANAMORFICO (figura 1.10) ############


# Este tipo de mapa tambien ha sido realizado con mas de un procedimiento.
# La ausencia (todavia) de un algoritmo funcional para generar cartogramas en el
# R, nos ha obligado a hacer uso de un programa externo para realizar dicha tarea.
# El programa utilizado ha sido el ScapeToad, que puede ser encontrado, de
# forma gratuita en la direccion: http://scapetoad.choros.ch/index.php

# Para realizar el mapa anamorfico (o cartograma) utilizamos el archivo
# ccaa.shp y la variable POBLACION. Una vez creado el cartograma, se ha guardado
# el resultado en el archivo denominado "cartograma.shp". 

# El ultimo paso ha sido abrir el archivo cartograma.shp en el R y 
# pedir para que el programa dibujara el mapa.

# Abre el archivo shape del cartograma creado por el programa ScapeToad
carto <- readShapeSpatial("cartograma.shp")

# Genera el mapa del cartograma
plot(carto, col="grey90", border = "black")





##################### Mapa Tematico - MAPA de FLUJOS (figura 1.11) ############################


# Paso 1: abre los archivos de mapas y los datos

# Abre el archivo de mapas de las Comunidades Autonomas
library(maptools)
ccaa <- readShapeSpatial("ccaa.shp", proj4string = CRS(proyeccion), IDvar = "SP_ID",repair=TRUE,force_ring=TRUE)

# Abre el archivo de mapas de los centroides de las CCAA
ccaa_cid <- data.frame(ccaa$CC_COD, coordinates(ccaa))
names(ccaa_cid) <- c("CC_COD", "X","Y")

# Abre el archivo con los datos de las migraciones internas en 2010 en Espana
# Fuente: INE - Estadisticas de variaciones residenciales interiores 2010.
migra_cyl <- read.csv("migra_cyl.csv", header=TRUE, sep=";")


# Paso 2 - Vincula las coordinadas geograficas de origen (Castilla y Leon) y
#         destino (demas comunidades) con los datos de migracion

# Obtiene las coordinadas geograficas del punto de origen, en este caso el 
# centroide de Castilla y Leon.
origen <- c(ccaa_cid$X[ccaa_cid$CC_COD==7], ccaa_cid$Y[ccaa_cid$CC_COD==7])

# Calcula la proporcion que cada CCAA representa del total de emigrantes
# este indicador sera util para definir el grosor de la linea de cada CCAA.
migra_cyl$wdt <- migra_cyl$migra /sum(migra_cyl$migra,na.rm=T)


# Genera las coordinadas de origen y destino para cada CA que ha recibido 
# migrantes de Castilla y Leon  
for (i in 1:nrow(migra_cyl)){
    # Define el punto de origen para todos los casos
    migra_cyl$orX[i] <- origen[1]
    migra_cyl$orY[i] <- origen[2]

    # Define el punto de destino para cada caso
    migra_cyl$desX[i] <- ccaa_cid$X[ccaa_cid$CC_COD==migra_cyl$cc_cod[i]]
    migra_cyl$desY[i] <- ccaa_cid$Y[ccaa_cid$CC_COD==migra_cyl$cc_cod[i]]
}


# Paso 3 - Genera el mapa con TODAS las CCAA que han recibido emigrantes de 
#          Castilla y Leon

# Mapea las fronteras de las comunidades autonomas
plot(ccaa)

# SETAS - la funcion arrows(.) permite dibujar setas segun un origen y destino especificados 
# en un mapa. Ojo! Para que funcione, el mapa debe haber sido ya dibujado. 
# Parametros de la funcion arrows(.):
#       X del origen
#       Y del origen
#       X del destino
#       Y del destino
#       demas parametros relacionados con el tamano, color y forma de la punta de la seta y
#       otros atributos. Para mas informacion - help(arrows)
arrows(migra_cyl$orX,migra_cyl$orY, migra_cyl$desX, migra_cyl$desY, length=0.1, lwd=1+(10*migra_cyl$wdt))



# Paso 3 - Genera el mapa con solamente las CCAA que han recibido mas de 50 emigrantes de 
#          Castilla y Leon

# Selecciona las CCAA que han recibido mas de 50 emigrantes
mas_50 <- subset(migra_cyl, migra >=50)

# dibuja las setas (del mismo modo en que el ejemplo anterior)
arrows(mas_50$orX,mas_50$orY, mas_50$desX, mas_50$desY, length=0.1, lwd=1+(10*mas_50$wdt))

# SETA CURVA 

# El R permite tambien hacer uso de setas curvas, lo que ayuda a evitar la 
# superposicion y hacer mas clara la representacion de los datos.

# Para eso se empleara la funcion curvedarrow(.) del paquete "diagram"
library(diagram)

# Vuelve a dibujar el mapa base.
plot(ccaa)

# Para cada una de las CCAA con mas de 50 emigrantes de CyL
for (i in 1:nrow(mas_50)){
  
  # Si representa el mayor flujo (valor maximo) - dibuja una seta recta y azul
  if (mas_50$migra[i] == max(mas_50$migra)){
      arrows(mas_50$orX[i],mas_50$orY[i], mas_50$desX[i], mas_50$desY[i], length=0.1,      lwd=1+(10*mas_50$wdt[i]), col="blue")
    }
  # Caso contrario: dibuja las setas curvas con el tamano segun su proporcion en el total de emigrantes
  else {
      curvedarrow(c(mas_50$orX[i],mas_50$orY[i]), c(mas_50$desX[i], mas_50$desY[i]), curve=0.1, lwd=1+(10*mas_50$wdt[i]), arr.pos=1,arr.width=0.1, arr.length=0.1)
    }
}
      
      
###############################################################################
############ FINAL DEL ARCHIVO DE EJEMPLOS - CAPITULO 1 #######################
###############################################################################
