#############################################################################################
#   
#   REPRESENTACION ESPACIAL Y MAPAS 
#    
#   por
#
#   Rodrigo Rodrigues-Silveira
#   rodrodr@gmail.com
#   
#    SEGUNDO CAPITULO - RESOLUCION DE LOS EJERCICIOS
#
#   Este codigo esta destinado a ser utilizado en el paquete estadistico R
#   R version 3.0.1 (Windows). Ademas, para que funcione, todos los archivos denominados
#   "material auxiliar" deben estar presentes en la carpeta:
#   C:\\CIS\\Material Auxiliar\\
#   En el caso de que el usuario desee utilizar otra carpeta, debe alterar el 
#   contenido de la variable "cdt" - carpeta de trabajo, mas abajo.
#   
#   Ultima revision: 19 de julio de 2013.
###########################################################################################



#####################################################################################################
############################### PREPARACION DEL AMBIENTE ############################################
#####################################################################################################

# Define la carpeta de trabajo
setwd("C:/CIS/Material Auxiliar/Shapes/")


######################################### EJERCICIO 1 ###############################################
############################### EJEMPLOS EMPLEADOS EN EL CAPITULO ###################################
#####################################################################################################

# Abrir el archivo Cap2.R y ejecutar el codigo-fuente acompanando el desarrollo.


######################################### EJERCICIO 2 ###############################################
##################################### ABRIR BASE DE DATOS ###########################################
#####################################################################################################

# Abre el archivo de datos
datos <- read.csv("voto_mun_08.csv", header=TRUE, sep=";")

# Exhibe el nombre de las variables
names(datos)

# Provee las estadisticas descriptivas de todas las variables contenidas en la base de datos
summary(datos)

# Guarda la base de datos en un archivo
write.table(datos, "municipales08.csv", sep=";")


######################################### EJERCICIO 3 ###############################################
################################### ANALISIS Y CASOS PERDIDOS #######################################
#####################################################################################################

# Paso 1: Analisis de los casos perdidos

# Carga el paquete VIM, necesario para el analisis de los casos perdidos
library(VIM)

# Ejecuta la funcion aggr(.) para analizar el patron de los casos perdidos
aggr(datos)

# Lista el nombre de variables de la base de datos
names(datos)

# Define los margenes para los graficos
par(mar=c(4,4,4,0.1))

# Ejecuta la funcion spineMiss(.) para averiguar si existe variacion sistematica de casos perdidos para diferentes niveles de otras variables
spineMiss(datos)
                                

# Paso 2: Imputacion de los valores perdidos

# Metodo - los K vecinos mas cercanos
datos <- kNN(datos)

######################################### EJERCICIO 4 ###############################################
##################################### AGREGACION DE CASOS ###########################################
#####################################################################################################

# Agrega las variables numericas (colunas 6 a 15 de la base de datos) para cada CA. La funcion utilizada es la suma.
agrega <- data.frame(aggregate(datos[6:15], by=list(datos$ccaa_nombre), FUN=sum))

# Guarda los resultados en el archivo agrega.csv
write.table(agrega, "agrega.csv", sep=";")

######################################### EJERCICIO 5 ###############################################
################################### CALCULO DE INDICADORES ##########################################
#####################################################################################################

# calcula el porcentaje de Votos Validos
agrega$vv <- agrega$votos_validos / agrega$votantes_tot * 100

# calcula el porcentaje de Votos en Blanco
agrega$vb <- agrega$votos_validos / agrega$votantes_tot * 100

# calcula el porcentaje de Votos Nulos
agrega$vn <-agrega$votos_validos / agrega$votantes_tot * 100

# calcula el porcentaje de Participacion en las elecciones
agrega$part <- agrega$votantes_tot/agrega$censados_tot * 100

# calcula el porcentaje de Votos en el PSOE
agrega$psoe_per <- agrega$psoe / agrega$votos_validos * 100

# calcula el porcentaje de Votos en el PP
agrega$pp_per <- agrega$pp / agrega$votos_validos * 100

# calcula el porcentaje de Votos en otros partidos
agrega$otros_per <-agrega$otros / agrega$votos_validos * 100

# Guarda los resultados en el archivo agrega.csv
write.table(agrega, "agrega.csv", sep=";")


######################################### EJERCICIO 6 ###############################################
################################### FUSION CON EL ARCHIVO SHAPE #####################################
#####################################################################################################

# Carga el paquete responsable de abrir los archivos shape
library(maptools)

# Abre el archivo de CCAA
ccaa <- readShapeSpatial("ccaa.shp")

# Abre el archivo de datos "agrega.csv" (despues de incluir la variable CC_COD con los mismos codigos que los existentes en el archivo de mapas)
agrega <- read.csv("agrega.csv", header=TRUE, sep=";")
names(agrega)[1] <- "NOMBRE_CA"

# introduce el codigo de las comunidades autonomas segun el orden con que se encuentran en el archivo Shape
agrega$CC_COD <- c(1,2,5,6,8,7,9,18,13,15,10,11,12,4,17,16,3,14)
row.names(agrega) <- row.names(agrega$CC_COD)

# Genera nuevos nombres de identificacion para los estados 
ccaa <- spChFIDs(ccaa, as.character(ccaa$CC_COD))

# Fusiona los dos archivos
ccaa <- SpatialPolygonsDataFrame(ccaa, agrega, match.ID=T)

# Exhibe los resultados para inspeccion
summary(ccaa)

#####################################################################################################
############################# FIN DEL ARCHIVO DE SOLUCION DE EJERCICIOS #############################
#####################################################################################################