#############################################################################################
#   
#   REPRESENTACION ESPACIAL Y MAPAS 
#    
#   por
#
#   Rodrigo Rodrigues-Silveira
#   rodrodr@gmail.com
#   
#   TERCER CAPITULO - RESOLUCION DE LOS EJERCICIOS
#
#   Este codigo esta destinado a ser utilizado en el paquete estadistico R
#   R version 3.0.1 (Windows). Ademas, para que funcione, todos los archivos denominados
#   "material auxiliar" deben estar presentes en la carpeta:
#   C:\\CIS\\Material Auxiliar\\
#   En el caso de que el usuario desee utilizar otra carpeta, debe alterar el 
#   contenido de la variable "cdt" - carpeta de trabajo, mas abajo.
#   
#   Ultima revision: 19 de julio de 2013.
###########################################################################################

#####################################################################################################
############################### PREPARACION DEL AMBIENTE ############################################
#####################################################################################################

# Define la carpeta de trabajo
setwd("C:/CIS/Material Auxiliar/shapes/")

par(mar=c(0,0,0,0.1))

# Carga en la memoria las funciones Cartograficas Auxiliares
source("carto_functions.R")


######################################### EJERCICIO 1 ###############################################
############################### EJEMPLOS EMPLEADOS EN EL CAPITULO ###################################
#####################################################################################################


# Abrir el archivo Cap3.R y ejecutar el codigo-fuente acompanando el desarrollo.


######################################### EJERCICIO 2 ###############################################
############################### ABRIENDO MAPAS Y CREANDO VISTAS #####################################
#####################################################################################################

# Carga el paquete responsable de abrir los archivos shape
library(maptools)

# Abre el archivo de regiones de Francia
fra_reg <- readShapeSpatial("FRA_adm1.shp")

# Abre el archivo de departamentos de Francia
fra_dep <- readShapeSpatial("fra_adm2.shp")

# Abre el archivo con los ferrocarriles de Francia
fra_rail <- readShapeSpatial("fra_rails.shp")

# Dibuja los ferrocarriles en azul
plot(fra_rail, col="blue")

# Dibuja los departamentos en rojo y con borda=2, sobrescriviendo el mapa de ferrocarriles (add=TRUE)
plot(fra_dep, border="red", lwd=2, add=TRUE)

# Dibuja las regiones en negro, sobrescribiendo los niveles anteriores.
plot(fra_reg, add=TRUE, lwd=2)


######################################### EJERCICIO 3 ###############################################
############################### ANADIENDO ELEMENTOS A UN MAPA #######################################
#####################################################################################################

# Abre el archivo de mapa de los estados mexicanos
est <- readShapeSpatial("mex_adm1.shp")

# Divide a los casos en cuatro franjas o intervalos
franjas <- quantile(est$POB24MSUP1, seq(0,1,0.25))

# Define cuatro colores
colores <- c("white", "darkslategray1", "dodgerblue", "blue4")

# Genera el mapa de la poblacion con estudios superiores
plot(est, col=colores[findInterval(est$POB24MSUP1, franjas, all.inside=TRUE)], border = "black")

# Genera el texto a ser utilizado en la leyenda del mapa 
leg <- genLeg(franjas)

# Genera la leyenda del mapa
legend("bottomleft",leg, fill=colores, title="% est. superiores")

# Anade el titulo segun las coordinadas obtenidas
text(-100.5552,35.51076, "% con estudios superiores")

# Anade la escala grafica de 500 kilometros (unit=1) segun las coordinadas obtenidas
scalebar(-99.68068, 12.4557, 500, unit=1)

# Anade la Rosa de los Vientos con las letras E- Este, N- Norte, O - Oeste, S- Sur
northarrow(-93.11567, 23.02525, 0.8,cex=1, letters=c("E", "N","O","S"))


######################################### EJERCICIO 4 ###############################################
##################################### CLASIFICACION DE DATOS ########################################
#####################################################################################################

# Carga el paquete classInt y calcula el tipo - intervalos iguales
library(classInt)

# Numero igual de casos - quantiles
int <- classIntervals(est$MED_CMILPU, 4, "quantile")

# Natural breaks
int <- classIntervals(est$MED_CMILPU, 4, "jenks")

# Desviacion estandar
int <- classIntervals(est$MED_CMILPU, 4, "sd")

# Funcion colorRampPalette(.) - genera una funcion (denominada col(.)) que permite crear 
# una secuencia continua y progresiva de colores para todos los elementos de una determinada variable.
col <- colorRampPalette(c("white","blue4"))
col_leg <- col(4)

# Establece los colores para lla desviacion estandar
diver <- colorRampPalette(c("blue","white","red3"))
col_leg <- diver(6)

# Relaciona cada caso a un color especifico por medio de la funcion findColours(.)
colores <- findColours(int, col_leg)
plot(est, col=colores, border = "black")
leg <- genLeg(int$brks)
legend("bottomleft",leg, fill=col_leg, title="Méd./100mil")


# Mapa de variable categorica
brk <- sort(unique(est$PART_GOB10))

# La misma funcion puede ser empleada para generar una paleta con tres o mas colores
multicol <- colorRampPalette(c("green","blue4","red3"))
colores <- multicol(4)

plot(est, col=colores[findInterval(est$PART_GOB10, brk, all.inside=FALSE)], border = "black")

legend("bottomleft",as.character(brk), fill=colores, title="Partidos")




######################################### EJERCICIO 5 ###############################################
############################ COMBINACION DE GRAFICOS EN UNA MISMA VENTANA ###########################
#####################################################################################################


# Carga el paquete classInt y calcula el tipo - intervalos iguales
library(classInt)

# Genera la matriz con dos lineas y tres columnas que servira de base para la combinacion de los graficos
m <- matrix(c(1,1,2,1,1,3),2,3, byrow=TRUE)

# Emplea la matriz para crear el layout deseado
layout(m)

# Natural breaks
int <- classIntervals(est$MED_CMILPU, 4, "jenks")


# Funcion colorRampPalette(.) - genera una funcion (denominada col(.)) que permite crear 
# una secuencia continua y progresiva de colores para todos los elementos de una determinada variable.
col <- colorRampPalette(c("white","blue4"))
col_leg <- col(4)


# Relaciona cada caso a un color especifico por medio de la funcion findColours(.)
colores <- findColours(int, col_leg)
plot(est, col=colores, border = "black")
leg <- genLeg(int$brks)
legend("bottomleft",leg, fill=col_leg, title="Méd./100mil")

# Genera el histograma
hist(est$MED_CMILPU, main="Histograma")

# Genera el Boxplot
boxplot(est$MED_CMILPU, main="Boxplot")


######################################### EJERCICIO 6 ###############################################
############################### GUARDANDO LOS RESULTADOS EN ARCHIVOS ################################
#####################################################################################################

# Define que el R abre un dispositivo de im?gen de tipo JPEG denominado "pop_mexico.jpg" con la altura de 1280 y largo de 2048.
jpeg("Pop_mexico.jpg", height=1280, width=2048)

# Carga el paquete classInt y calcula el tipo - intervalos iguales
library(classInt)

# Genera la matriz con dos lineas y tres columnas que servira de base para la combinacion de los graficos
m <- matrix(c(1,1,2,1,1,3),2,3, byrow=TRUE)

# Emplea la matriz para crear el layout deseado
layout(m)

# Natural breaks
int <- classIntervals(est$MED_CMILPU, 4, "jenks")


# Funcion colorRampPalette(.) - genera una funcion (denominada col(.)) que permite crear 
# una secuencia continua y progresiva de colores para todos los elementos de una determinada variable.
col <- colorRampPalette(c("white","blue4"))
col_leg <- col(4)


# Relaciona cada caso a un color especifico por medio de la funcion findColours(.)
colores <- findColours(int, col_leg)
plot(est, col=colores, border = "black")
leg <- genLeg(int$brks)
legend("bottomleft",leg, fill=col_leg, title="Méd./100mil")

# Genera el histograma
hist(est$MED_CMILPU, main="Histograma")

# Genera el Boxplot
boxplot(est$MED_CMILPU, main="Boxplot")

# Guarda los resultados de modo definitivo los resultados en el archivo
dev.off()

## OBS.: Repetir la misma operacion para las funciones bmp(.) y png(.)
# bmp("Pop_mexico.bmp", height=1280, width=2048)
# png("Pop_mexico.png", height=1280, width=2048)


#####################################################################################################
############################# FIN DEL ARCHIVO DE SOLUCION DE EJERCICIOS #############################
#####################################################################################################
