#############################################################################################
#   
#   REPRESENTACION ESPACIAL Y MAPAS 
#    
#   por
#
#   Rodrigo Rodrigues-Silveira
#   rodrodr@gmail.com
#   
#   CUARTO CAPITULO - RESOLUCION DE LOS EJERCICIOS
#
#   Este codigo esta destinado a ser utilizado en el paquete estadistico R
#   R version 3.0.1 (Windows). Ademas, para que funcione, todos los archivos denominados
#   "material auxiliar" deben estar presentes en la carpeta:
#   C:\\CIS\\Material Auxiliar\\
#   En el caso de que el usuario desee utilizar otra carpeta, debe alterar el 
#   contenido de la variable "cdt" - carpeta de trabajo, mas abajo.
#   
#   Ultima revision: 19 de julio de 2013.
###########################################################################################

#####################################################################################################
############################### PREPARACION DEL AMBIENTE ############################################
#####################################################################################################

# Define la carpeta de trabajo
setwd("C:/CIS/Material Auxiliar/Shapes")

par(mar=c(0,0,0,0.1))

# Carga en la memoria las funciones Cartograficas Auxiliares
source("carto_functions.R")


######################################### EJERCICIO 1 ###############################################
############################### EJEMPLOS EMPLEADOS EN EL CAPITULO ###################################
#####################################################################################################


# Abrir el archivo Cap4.R y ejecutar el codigo-fuente acompanando el desarrollo.




######################################### EJERCICIO 2 ###############################################
######################################## DE COROPLETAS ##############################################
#####################################################################################################

# Carga los paquetes necesarios
library(maptools)
library(classInt)
library(RColorBrewer)

# Abre los archivos 
mex <- readShapeSpatial("MEX_adm1.shp")
mundo <- readShapeSpatial("mundo.shp")

# Maneja el color
pale <- brewer.pal(4, "Greens")
inte <- classIntervals(mex$MED_CMILPU,n=4, style="jenks")
colores <- findColours(inte, pale)

# Construye el mapa
par(mar=rep(0.1,4))
plot(mundo, bg="lightblue",col="wheat1", xlim=bbox(mex)[1,],ylim=bbox(mex)[2,])
plot(mex, col=colores, add=T)

# Incluye la leyenda
breaks <- inte$brks
legs <- leglabs(format(breaks,decimal.mark=",", big.mark="."), under="<", over=">", between="a")
legend("bottomleft", legend=legs, fill=pale, bty="y")


######################################### EJERCICIO 3 ###############################################
##################################### DENSIDAD DE PUNTOS ############################################
#####################################################################################################


# Abre los archivos
est <- readShapeSpatial("Estados.shp")
aero <- readShapeSpatial("aeroportos.shp")

# A) Genera el mapa de puntos sencillo
plot(aero, xlim=bbox(est)[1,], ylim=bbox(est)[2,], pch=19, col="blue", cex=0.8)
plot(est, lwd=3, add=T)

# B) Representa los aeropuertos como aviones por medio de una TTF
windowsFonts(web=windowsFont("Wingdings"))
plot(est)
text(coordinates(aero), labels="Q", family="web", col="red", cex=0.5)
plot(est, lwd=3, add=T)

# C) Gradiente de color

pale <- brewer.pal(4, "Oranges")
inte <- classIntervals(aero$ALTITUDE,n=4, style="kmeans")
aero$col <- findColours(inte, pale)

plot(aero, xlim=bbox(est)[1,], ylim=bbox(est)[2,], pch=19, col=aero$col, cex=0.8)
plot(est, lwd=3, add=T)

# Incluye la leyenda
breaks <- inte$brks
legs <- leglabs(format(breaks,decimal.mark=",", big.mark="."), under="<", over=">", between="a")
legend("bottomleft", legend=legs, col=pale, pch=19, bty="n", title="Altitud (m.s.n.m.)")



######################################### EJERCICIO 4 ###############################################
################################# SIMBOLOS PROPORCIONALES ###########################################
#####################################################################################################

# Abre el archivo del DF y prepara el calculo
df <- readShapeSpatial("distrito federal.shp")
dfpt <- SpatialPointsDataFrame(df, data.frame(df))

# Define el tamano maximo del tamano del punto a ser representado
nmax <- 7

# Calculo del tamano de cada punto (con base en el tamano maximo de punto previamente definido para cex)
# Dos tipos:  
# - Matematico - cada cual es representado por su valor relativo de la variable en cuestion
# - Perceptual - Reduce el peso visual de los valores mas grandes
dfpt$size <- (dfpt$POP/max(dfpt$POP, na.rm=T))^0.5*nmax       # Matematico
dfpt$size <- (dfpt$POP/max(dfpt$POP, na.rm=T))^0.57*nmax      # Perceptual


#Calculo de los tamanos, de la legenda y del ordem de diseno de los elementos
legs <- sort(round(quantile(dfpt$POP,seq(0,1,1/3), na.rm=T),0),decreasing=T)
ri <- (legs/max(legs, na.rm=T))^0.5*nmax      # Matematico
ri <- (legs/max(legs, na.rm=T))^0.57*nmax # Perceptual


#Establece el orden de diseno de los mayores a los menores para evitar que la superposicion oculte puntos.
dfpt$plotOrder<- order(sort(dfpt$size, decreasing=T))


# A) Simbolos proporcionales simple.

par(mar=rep(0,4))
plot(df, col="wheat1")
plot(dfpt, cex=dfpt$size, col="black", bg="red", pch=21, add=T)
legs <- format(legs, big.mark=".")
legend("topleft", legend=legs[1:length(legs)-1], col="black", pt.bg="red",pch=21,pt.cex=ri[1:length(ri)-1], cex=1.2, bty="n", bg="transparent", y.intersp=1.6, x.intersp=1.2)


# B) Simbolos proporcionales y robos por mil habitantes

colores <- brewer.pal(4, "PuBu")
colpt <- min(colores)
int <- classIntervals(dfpt$ROB_CMIL, 4, "quantile")
breaks <- round(int$brks)
dfpt$col <- as.vector(findColours(int, colores))

plot(df)
plot(dfpt, col=dfpt$col, pch=19, cex=dfpt$size, add=T)

legs <- format(legs, big.mark=".")
legend("topleft", legend=legs[1:length(legs)-1], col="black", pt.bg=colpt,pch=21,pt.cex=ri[1:length(ri)-1], cex=1.2, bty="n", bg="transparent", y.intersp=1.6, x.intersp=1.2, title="Poblacion", adj=-0.1)

legsB <- leglabs(format(breaks,decimal.mark=",", big.mark="."), under="<", over=">", between="a")
legend("topright", legend=legsB, fill=colores, bty="n", title="Robos/mil hab.", cex=1.2)



######################################### EJERCICIO 5 ###############################################
##################################### MAPA DE ISOPLETAS #############################################
#####################################################################################################

# Carga el paquete neceario
library(spatstat)

# Genera la ventana utilizada como marco
spatstat.options(checkpolygons=FALSE)
sp_owin <- as.owin.SpatialPolygons(est)
spatstat.options(checkpolygons=TRUE)

# Calcula la densidad
coord <- coordinates(aero)
ptaero <- as.ppp(coord, W=sp_owin)
dens_aero <- density.ppp(ptaero, 1, diggle=TRUE, edge=TRUE, dimyx=648)

# Mapea los resultados
colores <- heat.colors(50)
plot(est)
plot(dens_aero, col=colores[50:1], add=TRUE)
plot(est, add=TRUE)

