#############################################################################################
#   
#   REPRESENTACION ESPACIAL Y MAPAS 
#    
#   por
#
#   Rodrigo Rodrigues-Silveira
#   rodrodr@gmail.com
#   
#    QUINTO CAPITULO - RESOLUCION DE LOS EJERCICIOS
#
#   Este codigo esta destinado a ser utilizado en el paquete estadistico R
#   R version 3.0.1 (Windows). Ademas, para que funcione, todos los archivos denominados
#   "material auxiliar" deben estar presentes en la carpeta:
#   C:\\CIS\\Material Auxiliar\\
#   En el caso de que el usuario desee utilizar otra carpeta, debe alterar el 
#   contenido de la variable "cdt" - carpeta de trabajo, mas abajo.
#   
#   Ultima revision: 19 de julio de 2013.
###########################################################################################


#####################################################################################################
############################### PREPARACION DEL AMBIENTE ############################################
#####################################################################################################

# Establece la carpeta de trabajo
setwd("C:/CIS/Material Auxiliar/Shapes/")

# Reduce los margenes de la figura
par(mar=c(0,0,0,0.1))

# carga el paquete maptools
library(maptools)



######################################### EJERCICIO 1 ###############################################
################## Creacion de objetos espaciales por subconjunto o atributos #######################
#####################################################################################################


# A - toponimos solamente de Leon

# Abre el archivo de mapa de los toponimos de la provincia de Leon
topo_prov_leon <- readShapeSpatial("topo_prov_leon.shp")

# Selecciona solamente a los toponimos del municipio de Leon (INE_MUN==24089)
topo_leon <- topo_prov_leon[topo_prov_leon$INE_MUN==24089,]

# B- Mapa de puntos de las escuelas
tipos_esc <- c("ESCUELA","INSTITUTO", "UNIVERSIDAD","COLEGIO", "GUARDERIA")

# Genera un vector (denominado nid) con la identificacion de los casos que corresponden a alguna de las opciones contenidas en el factor "tipos_esc"
id <- sapply(tipos_esc, function(x){ agrep(x, topo_leon$TEXTO)})
nid <- sort(unique(c(do.call("cbind",id))))

# Genera un nuevo objeto que contiene solamente los casos identificados como escuelas
esc_leon <- topo_leon[nid,]

# C - Mapa de puntos de los hospitales

# Repite los mismos procedimientos anteriores, pero ahora para el caso de los hospitales y otras infraestructuras espaciales
tipos_hosp <- c("CENTRO DE SALUD","HOSPITAL", "SALUD","CLINICA", "HIGIENE", "SANATORIO", "CRUZ ROJA")
id <- sapply(tipos_hosp, function(x){ agrep(x, topo_leon$TEXTO)})

# Otro modo de crear una variable a partir de una lista es la funcion unlist(.)
nid <- sort(unique(unlist(id)))
hosp_leon <- topo_leon[nid,]

# D - generacion del mapa final

# Abre el archivo de mapa de los sectores censales del estado de Bahia en Brasil
leon_manz <- readShapeSpatial("leon_manzanas.shp")

# Genera el mapa de las manzanas de Leon
plot(leon_manz)

# Genera el mapa con los resultados
plot(hosp_leon, pch=19, col="red", add=TRUE)
plot(esc_leon, pch=17, col="blue", cex=0.7,add=TRUE)



######################################### EJERCICIO 2 ###############################################
############# Geocodificacion de direcciones y creacion de mapas de puntos ##########################
#####################################################################################################


## A - Geocodificacion de las direcciones de los museos de Leon

museos <- read.csv("museos.csv", header=TRUE, sep=";")

View(museos)

# OBS: Repetir el procedimiento para cada museo

# Carga la biblioteca XML - requerida para obtener las coordinadas geograficas
library(XML)

for(i in 1:10){

# Define la direccion que se desea geocodificar
direccion <- paste(museos$direccion[i], ",", museos$cod_post[i], ", Leon, Castilla y Leon, Espana", sep=" ")

# Establece la raiz de la direccion Web (URL) de la aplicacion de geocodificaion de Google Maps
raiz <- "http://maps.googleapis.com/maps/api/geocode/xml"

# Genera la direccion web a partir de la direccion informada y busca las informaciones en Google Maps
# address= para la direccion a ser buscada
# &sensor= false define que no se utiliza un aparato con sensor
# &language=es  define el espanol como lengua base de la busqueda
web <- paste(raiz, "?address=", direccion, "&sensor=false&language=es", sep = "")

# La funcion URLencode(.) convierte la direccion URL enviada en el codigo apropiado para la internet
web <- URLencode(web)

# La funcion xmlTreeParse(.) obtiene los datos de tipo xml para que puedan ser procesados
base_res <- xmlTreeParse(web, useInternal=TRUE)

# Extrae la longitud y latitud de la direccion deseada
long <- sapply(getNodeSet(base_res, "/GeocodeResponse/result/geometry/location/lng"), function(el) xmlValue(el))
lati <- sapply(getNodeSet(base_res, "/GeocodeResponse/result/geometry/location/lat"),function(el) xmlValue(el))

# Retorna los valores de la longitud y latitud obtenidos
print(long)
print(lati)

# Anade los resultados al objeto data.frame museos
museos$lng[i] <- long[1] # el indice [1] se usa para cuando se obtiene mas de una coordenada
museos$lat[i] <- lati[1] # el indice [1] se usa para cuando se obtiene mas de una coordenada

}

# convierte los resultados para el formato numerico
museos$lng <- as.double(museos$lng)
museos$lat <- as.double(museos$lat)

# Inspecciona los resultados
View(museos)

# B - conversion de las coordinadas a un objeto de tipo Spatial Points
coordinates(museos) <- ~lng+lat

# Anade el mapa de puntos al grafico generado en el ejercicio anterior
plot(museos, pch=19, col="darkgreen", cex=1, add=TRUE)


# C - Guarda los resultados en el archivo de tipo Shape
writeSpatialShape(museos, "leon_museos.shp")


# D - Realiza la geocodificacion inversa

# Carga la biblioteca XML - requerida para obtener las coordinadas geograficas
library(XML)

# latitud y longitud aleatorias de las cuales se desean obtener informacion
long <- -5.5717520
lati <- 42.5998764
  
long <- -5.5666767
lati <- 42.5972169

long <- -5.5665780
lati <- 42.5993889


# Establece la raiz de la direccion Web (URL) de Google Maps
raiz <- "http://maps.googleapis.com/maps/api/geocode/xml"

# Genera la direccion web a partir de cada par de coordinadas y busca las informaciones en Google Maps
# latlng= latitud , longitud
web <- paste(raiz, "?latlng=", lati, ",", long, "&sensor=false&language=es", sep = "")
web <- URLencode(web)
base_res <- xmlTreeParse(web, useInternal=TRUE)

# Obtiene una lista con las informaciones de direccion
res <- sapply(getNodeSet(base_res, "/GeocodeResponse/result/address_component/long_name"), function(el) xmlValue(el))

# Genera una lista no repetida con las informaciones completas de la direccion perteneciente a esas coordinadas geograficas
unique(res)



######################################### EJERCICIO 3 ###############################################
########################## Calculo de Area y Matrices de Vecindad ###################################
#####################################################################################################


# A calculo del Area de las manzanas de Leon


# Carga los paquetes PBSMapping y rgdal, necesarios para la realizacion de las operaciones geograficas
library(PBSmapping)
library(rgdal)

# Cambia la proyeccion geografica para una UTM que permite convertir la unidad de mensuracion del mapa de grados decimales a kilometros
leon_km <- leon_manz
proj4string(leon_km) <- CRS("+proj=utm +zone=30 +datum=WGS84 +units=km")

# Convierte los limites de los distritos de Salvador a un objeto de tipo PolySet 
sv_leon_ps <- SpatialPolygons2PolySet(leon_km)

# Calcula el Area de cada uno de los distritos
leon_km$area <- calcArea(sv_leon_ps, rollup=1)$area


# B - calcula la mayor distancia entre hospitales y escuelas

# Carga el paquete spatstat, necesario para la identificacion del vecino mas cercano
library(spatstat)

# Identifica los vecinos mas cercanos de dos tipos distintos de puntos
# En este ejemplo seran los hospitales mas cercanos a cada escuela
dist_km <- spDists(esc_leon, hosp_leon, longlat=TRUE)

# Obtiene la mayor distancia entre los dos tipos
max(dist_km)


# C - obtiene una matriz de distancias entre escuelas y manzanas

# Calcula la matriz de distancias en kilometros entre escuelas y manzanas
dist_esc <- spDists(esc_leon, leon_manz, longlat=TRUE)

# Establece que valores uno para los casos que sean menores o iguales a 500 metros (y cero a los demas)
dist_esc[dist_esc > 0.25] <- 0
dist_esc[dist_esc <=0.25 & dist_esc >0] <- 1

# D - Generacion de la variable en la base de datos

# Genera una variable en el objeto "leon_manz" que indica cuales manzanas se encuentra hasta 250 metros de una escuela.
# genera una secuencia con el numero total de escuelas
count <- seq(1:54)

# Obtiene, para cada elemento de la variable "count", todas las manzanas que poseen el valor 1
id <- sapply(count, function(x){ which(dist_esc[x,]==1)})

# Genera una variable "nid" con la identificacion - unica y ordenada de los menores a los mayores valores - de las manzanas con valor 1.
nid <- sort(unique(unlist(id)))

# Creva una variable de identificacion de los casos con hasta 250m de distancia con cero para todos los casos
leon_manz$d250m <- 0

# Cambia el valor de la variable anterior para 1 en los casos en que se encuentren hasta 250 metros.
leon_manz$d250m[nid] <- 1


# E - Generacion del mapa identificando las manzanas hasta 250m y las escuelas.
colores <- c("white", "orange")

# Anade a las escuelas como circulos en azul
plot(leon_manz, col=colores[findInterval(leon_manz$d250m, c(0,1))], border="lightgray")
plot(esc_leon, pch=19, col="blue", cex=0.5, add=TRUE)


######################################### EJERCICIO 4 ###############################################
############################ Operaciones geograficas - Interseccion #################################
#####################################################################################################


# Abre el archivo de mapa de los codigos postales de la provincia de Leon
cp_leon <- readShapeSpatial("leon_codpost.shp")
proj4string(cp_leon) <-  CRS("+proj=utm +zone=30 +datum=WGS84 +units=km")

# Genera el mapa con los codigos postales del municipio de Leon

# Anade las manzanas al mapa
plot(cp_leon, border="transparent")
plot(leon_manz, add=TRUE)
plot(cp_leon, border="red", add=TRUE)


# Remover el codigo postal 24010 (el que es enorme y esta casi vacio) y anadir el resultado al mapa
cp_leonB <- cp_leon[cp_leon$COD_POSTAL != "24010",]
plot(cp_leonB, border="blue", add=TRUE)

# llevar a cabo la interseccion
library(PBSmapping)

# conversion de los objetos al tipo Polyset
leon_manz.ps <- SpatialPolygons2PolySet(leon_manz)
leon_cp.ps <- SpatialPolygons2PolySet(cp_leonB)

# Operacion de interseccion y conversion a objeto de tipo spatial polygons
int <- joinPolys(leon_manz.ps, leon_cp.ps, operation="INT")
int <- PolySet2SpatialPolygons(int, close_polys=TRUE)

# Genera el mapa con los resultados
plot(int)
plot(cp_leonB, border="red", add=TRUE)



######################################### EJERCICIO 5 ###############################################
############################## Analisis de patrones de puntos #######################################
#####################################################################################################


# A densidad de Kernel de las escuelas de y hospitales de Leon
# Carga el paquete spatstat
library(spatstat)

# Crea una ventana de observacion con las fronteras del municipio de Salvador
spatstat.options(checkpolygons=FALSE)
wleon <- as.owin(cp_leonB)
spatstat.options(checkpolygons=TRUE)

# Genera un objeto de puntos con las escuelas dentro de la ventana de observacion creada anteriormente 
p_esc <- ppp(esc_leon$coords_x1, esc_leon$coords_x2, window=wleon)

# Genera la densidad de los puntos para las escuelas de Salvador
dens_esc <- density.ppp(p_esc, 0.004, diggle=TRUE, edge=TRUE, dimyx=648)

# Define una escala progresiva de colores calientes con 100 gradaciones
colores <- heat.colors(100)

# Genera el mapa del limite de Salvador
plot(cp_leonB)

# Genera el mapa isopletico de los resultados
plot(dens_esc, col=colores[100:1], add=TRUE)
plot(esc_leon, col="blue", pch=19, cex=0.5, add=TRUE)

# Superpone las fronteras de los distritos de Salvador
plot(cp_leonB, add=TRUE)

# Nota: la funcion persp.im(.) permite generar el mismo mapa en 3D, representando la densidad como elevacion.
par(mar=c(2,2,2,0.1))
persp.im(dens_esc, colmap=colores[100:1], theta = 20, phi = 60, main= "Densidad de escuelas (Leon)", zlab="Densidad", xlab="longitud", ylab="latitud")



# B - focos de incidencia de las escuelas

# Genera el mapa del limite de Salvador
par(mar=c(0,0,0,0.1))
plot(cp_leonB)

# Genera los focos de incidencia
sharp <- sharpen(as.ppp(esc_leon))

# Mapea los focos de incidencia
plot(sharp, bg="red", fg="transparent", main="", bty="n", add=TRUE)

# Genera el mapa de puntos con las escuelas
plot(esc_leon, pch=19, col="darkgreen", cex=0.6, add=TRUE)

# Sobrepone el mapa con los distritos
plot(cp_leonB, add=TRUE)



# C - Diagrama de Voronoi de los hospitales

# Carga el paquete PBSmapping, necesario para las operaciones geograficas deseadas
library(PBSmapping)

# Extrae las coordinadas de los hospitales de Salvador
coord <- data.frame(coordinates(hosp_leon))
names(coord) <- c("X","Y")

# Crea el mapa de los limites de Leon
cp_leonB$lim <- 1
leon_lim <- unionSpatialPolygons(cp_leonB, cp_leonB$lim)

# Calcula el diagrama de Voronoi
bb_leon <- bbox(leon_lim)
voro <- calcVoronoi(coord, xlim=bb_leon[1,], ylim=bb_leon[2,])

# Convierte los resultados en un PolySet y le confiere la projeccion "UTM" y la zona 24, donde se encuentra Salvador de Bahia
voro.ps <- as.PolySet(voro, projection = "UTM", zone ="30")


# Convierte los limites del municipios de Salvador a un PolySet 
leon <- SpatialPolygons2PolySet(leon_lim)

# Lleva a cabo la interseccion entre el diagrama de Voronoi y las fronteras de Salvador
voro.ps <- joinPolys(voro.ps, leon, operation="INT")

# Convierte los resultados en un objeto Spatial Polygons
voro_sppoly <- PolySet2SpatialPolygons(voro.ps, close_polys=TRUE)

# Genera una representacion visual de la operacion
plot(voro_sppoly)
plot(hosp_leon, pch=19, col="red", cex=0.8, add=TRUE)




######################################### EJERCICIO 6 ###############################################
################################ Exportacion de los resultados ######################################
#####################################################################################################


# carga el paquete rgdal
library(rgdal)

# exporta los objetos espaciales esc_leon y hosp_leon al Google Earth

# Escuelas
kmlPoints(esc_leon, kmlfile="leon_escuelas.kml", kmlname="Escuelas de Leon", "Escuelas de Leon", description=esc_leon$TEXTO)

# Hospitales
kmlPoints(hosp_leon, kmlfile="leon_hospitales.kml", kmlname="Hospitales de Leon", "Hospitales de Leon", description=hosp_leon$TEXTO)

# transforma las coordinadas para las adecuadas al Google Earth
proj <- CRS("+proj=longlat +datum=WGS84")
cpleon <- spTransform(cp_leonB, proj)

# exporta los poligonos con los codigos postales a un archivo .kml
writeOGR(cpleon, dsn="Leon_codpost.kml", layer="Codigos postales de Leon", driver="KML", overwrite_layer=TRUE)



#####################################################################################################
############################# FIN DEL ARCHIVO DE SOLUCION DE EJERCICIOS #############################
#####################################################################################################


