#############################################################################################
#   
#   REPRESENTACI?N Y ANALISIS EXPLORATORIO ESPACIAL DE DATOS 
#    
#   por
#
#   Rodrigo Rodrigues-Silveira
#   rodrodr@gmail.com
#   
#   FUNCIONES CARTOGR?FICAS AUXILIARES
#
#   Este c?digo est? destinado a ser utilizado en el paquete estad?stico R
#   R version 2.13.2 (Windows). Adem?s, para que funcione, todos los archivos denominados
#   "material auxiliar" deben estar presentes en la carpeta:
#   C:\\CIS_Manual\\Material Auxiliar\\
#   En el caso de que el usuario desee utilizar otra carpeta, debe alterar el 
#   contenido de la variable "cdt" - carpeta de trabajo, m?s abajo.
#
#   La funci?n genLeg() es de desarrollo propio.
#   Las funciones northarrow() y scalebar() son adaptaciones del c?digo publicado en:
#
#         Tanimura, Susumu, Chusi Kuroiwa, and Tsumotu Mizota. 2007. "Auxiliary Cartographic 
#         Functions in R: North Arrow, Scale Bar, and Label with a Leader Arrow." Journal of 
#         Statistical Software 19 (Code Snippet 1): 1-8.
#   
#   ?ltima revisi?n: 04 de noviembre de 2011.
###########################################################################################


# Genera Leyenda del mapa
genLeg <- function(brk, simb="-", dec=1, dec_coma=FALSE, sep_miles=FALSE) { 

leg <- vector()

dec_s <-""
mil_s <-""

if (dec_coma==TRUE){
   dec_s <- ","
   if (sep_miles==TRUE) mil_s <- "."
}

if (dec_coma==FALSE) {
  dec_s <-"."
  if (sep_miles==TRUE) mil_s <- ","
}      

for (i in 1:length(brk)-1){
  
  leg[i] <- c(paste(format(round(brk[i],dec), big.mark=mil_s, decimal.mark=dec_s), simb,      
                    format(round(brk[i+1],dec), big.mark=mil_s, decimal.mark=dec_s), sep=" "))
}

return(leg)

}


# Seta del Norte

northarrow <- function(loc,size,bearing=0,cols,cex=1, letters=c("E","N","W","S"), ...) {

# checking arguments
if(missing(loc)) stop("loc is missing")
if(missing(size)) stop("size is missing")

if (as.integer(loc[1])>2) size <- size*100000

# default colors are white and black
if(missing(cols)) cols <- rep(c("white","black"),8)
# calculating coordinates of polygons
radii <- rep(size/c(1,4,2,4),4)
x <- radii[(0:15)+1]*cos((0:15)*pi/8+bearing)+loc[1]
y <- radii[(0:15)+1]*sin((0:15)*pi/8+bearing)+loc[2]
# drawing polygons
for (i in 1:15) {
x1 <- c(x[i],x[i+1],loc[1])
y1 <- c(y[i],y[i+1],loc[2])
polygon(x1,y1,col=cols[i])
}
# drawing the last polygon
polygon(c(x[16],x[1],loc[1]),c(y[16],y[1],loc[2]),col=cols[16])
# drawing letters
b <- letters 
for (i in 0:3) text((size+par("cxy")[1])*cos(bearing+i*pi/2)+loc[1],
(size+par("cxy")[2])*sin(bearing+i*pi/2)+loc[2],b[i+1],
cex=cex)
}


# Escala grafica


scalebar <- function(loc,length,unit=1,division.cex=.8, lwd=1,...) {
  
  if(missing(loc)) stop("loc is missing")
  if(missing(length)) stop("length is missing")
  
  label <- c(0,length/c(4,2,4/3,1))
  
  if (unit==1) degrees <- length/111.325 # Kilometers - default
  if (unit==2) degrees <- length/(68.71) # Miles
  if (unit==3) degrees <- length/111325 # Meters
  
  length <- degrees
  
  scale_lab <- c("km", "miles", "meters")
  unit_c <- scale_lab[unit]
  
  x <- c(0,length/c(4,2,4/3,1),length*1.1)+loc[1]
  y <- c(0,length/(10*3:1))+loc[2]
  
  cols <- rep(c("black","white"),2)
  
  for (i in 1:4) rect(x[i],y[1],x[i+1],y[2],col=cols[i],lwd=lwd)
  
  for (i in 1:5){ 
    if (i == 1 | i == 3 | i==5)  segments(x[i],y[2],x[i],y[4], lwd=lwd)
    else segments(x[i],y[2],x[i],y[3], lwd=lwd)
  }
  
  labels <- label[c(1,3)]
  labels <- append(labels,paste(label[5],unit_c))
  text(x[c(1,3,5)],(y[4]+(length/10)),labels=labels,adj=0,cex=division.cex)
}

